/*
Calcolo della trasposta di una matrice n x n
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h> /*bisogna compilare con la flag -lm per includere la libreria
                    libm.so*/


/*Dichiarazione delle funzioni*/
void inserisci_matrice(int **a, int n); /*per inserimento a mano*/
void genera_matrice(int **a, int n, int range); /*per inserimento casuale*/
void stampa_matrice(int **a,int n); /*per la stampa*/


int main(int argc, char* argv[])
{   
    int n;
    int scelta;
    int i,j;
    int test_ciclo;
    int ret;
    int range_elem;
    int temp;
    int **ma;

    /*ciclo per ripetere il menu*/
    test_ciclo=1;
    while(test_ciclo)   
    {
        printf("\n********** MENU ***********\n");
        printf("\n1 INSERIRE ELEMENTI MATRICE");
        printf("\n2 GENERARE ELEMENTI MATRICE");
        printf("\n0 USCITA\n");
        printf("\nScegli un'opzione :");
        
        /*lettura della scelta fatta dall'utente.
        La scanf ritorna 0 nel caso in cui non abbia
        letto il valore richiesto nella stringa di
        formattazione*/
        ret=scanf("%d", &scelta);

        /*controllo che l'utente abbia scelto solo 0, 1, 2*/
        if(scelta<=0 || scelta>2 || ret==0)
            test_ciclo=0;
        else
        {   
            printf("\nInserisci il rango della matrice: ");
            scanf("%d",&n);
            
            if (n < 1) 
            {
               printf("Errore: il rango deve essere un intero positivo.\n");
                 exit(1);
            } 
            
            
            /* 
            Allocazione memoria per la matrice.
            Prima allochiamo una colonna di puntatori
            a interi. Ogni elemento della colonna
            sara' a sua volta un puntatore a interi e servira' per puntare
            allo spazio che conterra' una riga di interi
            */
            
            ma = (int**)malloc(sizeof(int)*n);  
    
            if (ma == NULL) {
                printf("Allocazione memoria fallita\n");
                exit(1);
            }   
            
            for(i=0;i<n;i++) {
                ma[i] = (int*)malloc(sizeof(int)*n);
                if (ma[i] == NULL) {
                    printf("Allocazione memoria fallita per l'elemento %d\n", i);
                    exit(1);
                }
            }   
                        
            if(scelta==1) /*se l'utente sceglie 1 chiamo la funzione inserisci_matrice*/
            {
                inserisci_matrice(ma,n);
            }
            else /*se l'utente sceglie 2 chiamo la funzione genera_matrice*/
            {
                printf("\nInserisci il range degli elementi (max=%d) : ",RAND_MAX);
                scanf("%d",&range_elem);
                genera_matrice(ma,n,range_elem);
            }

            printf("\nINPUT :\nA :\n");
            stampa_matrice(ma,n);    

            /* calcola la trasposta */                
            for (i=0;i<n;i=i+1) /*indice riga di ma*/
                for (j=i+1;j<n;j=j+1)
                /*indice colonna di ma
                Parto dall'elemento successivo alla diagonale*/
                {
                    /* SWAP */
                    temp=ma[i][j];
                    ma[i][j]=ma[j][i];
                    ma[j][i]=temp;
                }

                printf("\nTRASPOSTA di A:\n\n");
                stampa_matrice(ma,n);

        }

    }
    
    /* libero la memoria allocata */
    
    for(i=0;i<n;i++)
        free(ma[i]);

    free(ma);
    
}

/*definizione della funzione genera_matrice
Parametri: matrice, rango, range dei valori*/
void genera_matrice(int **a, int n, int range)
{   
    int i,j;
    double num_random;
    
    srand(time(NULL));

    for(i=0;i<n;i=i+1)
    {
        for (j=0;j<n;j=j+1)
        {   /*rand() restituisce un numero random tra 0 e RAND_MAX*/
            num_random = rand();
            /*normalizzazione del numero generato in modo
            che sia compreso tra 0 e range*/
            a[i][j]= (int)floor((num_random / RAND_MAX) * range);
        }
    }
        
}

/*definizione della funzione stampa_matrice
Parametri: matrice, rango*/
void stampa_matrice(int **a,int n)
{
    int i,j;

    for(i=0;i<n;i=i+1)
    {
        for (j=0;j<n;j=j+1)
        {
            printf("%5d ",a[i][j]);
        }
        printf("\n");
    }
        
            

}

/*definizione della funzione inserisci_matrice
Parametri: matrice, rango*/
void inserisci_matrice(int **a, int n)
{
    int i,j,ret;

    for(i=0;i<n;i=i+1)
    {
        for (j=0;j<n;j=j+1)
        {
            printf("\nInserire elemento [%d][%d] :",i,j);
            ret = scanf("%d",&a[i][j]);
            if (ret==0)
                a[i][j]=0;
        }
        printf("\n");
    }
}
